# ** header v3.0
# This file is a part of the CaosDB Project.
#
# Copyright (C) 2021 Indiscale GmbH <info@indiscale.com>
# Copyright (C) 2021 Florian Spreckelsen <f.spreckelsen@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program. If not, see
# <https://www.gnu.org/licenses/>.
#
# ** end header
#

module Transaction

using CaosDB

"""
Struct containing a pointer to the wrapped cpp transaction
object. Meant for internal use; call
`CaosDB.Transaction.create_transaction` to create a transaction
object.
"""
mutable struct _Transaction
    wrapped_transaction::Ptr{Cvoid}
    function _Transaction(managed_by_julia::Bool = false)
        trans = new()
        if managed_by_julia
            function f(t)
                ccall(
                    (:caosdb_transaction_delete_transaction, CaosDB.library_name),
                    Cint,
                    (Ref{_Transaction},),
                    Ref{_Transaction}(t),
                )
            end
            finalizer(f, trans)
        end
        return trans
    end
end

end # Transaction
